/***
 *       _____                         _   
 *      / ____|                       | |  
 *     | (___     ___   _ __     ___  | |_ 
 *      \___ \   / _ \ | '_ \   / _ \ | __|
 *      ____) | |  __/ | | | | |  __/ | |_ 
 *     |_____/   \___| |_| |_|  \___|  \__|
 *         (C) 2016 Senet, Inc                                
 *                                         
 */
#ifdef MTDOT_EVB
#include "board.h"

#include "MMA845x.h"
#include "MPL3115A2.h"
#include "ISL29011.h"
#include "NCP5623B.h"
#include "DOGS102.h"
#include "font_6x8.h"
#include "MultiTech_Logo.h"

//DigitalIn mDot02(PA_2);  //  GPIO/UART_TX
//DigitalOut mDot03(PA_3); //  GPIO/UART_RX
//DigitalIn mDot04(PA_6);  //  GPIO/SPI_MISO
//DigitalIn mDot06(PA_8);  //  GPIO/I2C_SCL
//DigitalIn mDot07(PC_9);  //  GPIO/I2C_SDA

InterruptIn mDot08(PA_12);  //  GPIO/USB       PB S1 on EVB
InterruptIn mDot09(PA_11);  //  GPIO/USB       PB S2 on EVB

//DigitalIn mDot11(PA_7);   //  GPIO/SPI_MOSI

InterruptIn mDot12(PA_0);    //  GPIO/UART_CTS  PRESSURE_INT2 on EVB
DigitalOut  mDot13(PC_13,1); //  GPIO           LCD_C/D
InterruptIn mDot15(PC_1);    //  GPIO           LIGHT_PROX_INT on EVB
InterruptIn mDot16(PA_1);    //  GPIO/UART_RTS  ACCEL_INT2 on EVB
DigitalOut mDot17(PA_4,1);   //  GPIO/SPI_NCS   LCD_CS on EVB

//DigitalIn mDot18(PA_5);    //  GPIO/SPI_SCK

//DigitalInOut mDot19(PB_0,PIN_INPUT,PullNone,0); // GPIO         PushPull LED Low=Red High=Green set MODE=INPUT to turn off
AnalogIn mDot20(PB_1);         //  GPIO          Current Sense Analog in on EVB
Serial debugUART(PA_9, PA_10); // mDot debug UART
//Serial mDotUART(PA_2, PA_3); // mDot external UART mDot02 and mDot03
I2C mDoti2c(PC_9,PA_8);	// mDot External I2C mDot6 and mDot7

SPI mDotspi(PA_7,PA_6,PA_5); // mDot external SPI mDot11, mDot4, and mDot18

static MMA845x_DATA   accel_data   = 0;
static MPL3115A2_DATA baro_data    = 0;
static MMA845x*       evbAccel     = 0;
static MPL3115A2*     evbBaro      = 0;
static ISL29011*      evbAmbLight  = 0;
static NCP5623B*      evbBackLight = 0;
static DOGS102*       evbLCD       = 0;
static Thread*        thread_3     = 0;

uint8_t  result, pckt_time=100;
char     data;

// flags for pushbutton debounce code
bool pb1_low = false;
bool pb2_low = false;

void pb1ISR(void);
void pb2ISR(void);
void pb1_debounce(void const *args);
void pb2_debounce(void const *args);
void config_pkt_xmit (void const *args);

CBoardEVB::CBoardEVB()
{
	boardPtr = this;
}


EBoardStatus CBoardEVB::init()
{
	CBoard::init();

	// Setting up LED1 as activity LED
	mDotPtr->setActivityLedPin(PB_0);
	mDotPtr->setActivityLedEnable(true);

    // threads for de-bouncing pushbutton switches
	static Thread thread_1(pb1_debounce);
    static Thread thread_2(pb2_debounce);

    thread_3 = new Thread(config_pkt_xmit); // start thread that sends LoRa packet when SW2 pressed

    evbAccel     = new MMA845x(mDoti2c,MMA845x::SA0_VSS); // setup Accelerometer
    evbBaro      = new MPL3115A2(mDoti2c); // setup Barometric sensor
    evbAmbLight  = new ISL29011(mDoti2c); // Setup Ambient Light Sensor
    evbBackLight = new NCP5623B(mDoti2c); // setup backlight and LED 2 driver chip
    evbLCD       = new DOGS102(mDotspi, mDot17, mDot13); // setup LCD

    /*
     *  Setup SW1 as program stop function
     */
    mDot08.disable_irq();
    mDot08.fall(&pb1ISR);

    /*
     *  need to call this function after rise or fall because rise/fall sets
     *  mode to PullNone
     */
    mDot08.mode(PullUp);

    mDot08.enable_irq();

    /*
     *  Setup SW2 as packet time change
     */
    mDot09.disable_irq();
    mDot09.fall(&pb2ISR);

    /*
     *  need to call this function after rise or fall because rise/fall sets
     *  mode to PullNone
     */
    mDot09.mode(PullUp);

    mDot09.enable_irq();

    /*
    * Setting other InterruptIn pins with Pull Ups
    */
    mDot12.mode(PullUp);
    mDot15.mode(PullUp);
    mDot16.mode(PullUp);

    printf("font table address %p\n\r",&font_6x8);
    printf("bitmap address %p\n\r",&MultiTech_Logo);

    // Setup and display logo on LCD
    evbLCD->startUpdate();

    evbLCD->writeBitmap(0,0,MultiTech_Logo);

    sprintf(txtstr,"MTDOT");
    evbLCD->writeText(24,3,font_6x8,txtstr,strlen(txtstr));
    sprintf(txtstr,"Evaluation");
    evbLCD->writeText(24,4,font_6x8,txtstr,strlen(txtstr));
    sprintf(txtstr,"Board");
    evbLCD->writeText(24,5,font_6x8,txtstr,strlen(txtstr));

    evbLCD->endUpdate();

    pckt_time = 10;

    return Board_Ok;
}

EBoardStatus CBoardEVB::start()
{
	osDelay(200);
	evbBackLight->setPWM(NCP5623B::LED_3,16); // enable LED2 on EVB and set to 50% PWM

	// sets LED2 to 50% max current
	evbBackLight->setLEDCurrent(16);

	printf("Start of Test\n\r");

	osDelay (500); // allows other threads to process
	printf("shutdown LED:\n\r");
	evbBackLight->shutdown();

	osDelay (500); // allows other threads to process
	printf("Turn on LED2\n\r");
	evbBackLight->setLEDCurrent(16);

	data = evbAccel->getWhoAmI();
	printf("Accelerometer who_am_i value = %x \n\r", data);

	result = evbAccel->getStatus();
	printf("status byte = %x \n\r", result);

	printf("Barometer who_am_i check = %s \n\r", evbBaro->testWhoAmI() ? "TRUE" : "FALSE");

	result = evbBaro->getStatus();
	printf("status byte = %x \n\r", result);

	/*
	*  Setup the Accelerometer for 8g range, 14 bit resolution, Noise reduction off, sample rate 1.56 Hz
	*  normal oversample mode, High pass filter off
	*/
	evbAccel->setCommonParameters(MMA845x::RANGE_8g,MMA845x::RES_MAX,MMA845x::LN_OFF,
			MMA845x::DR_1_56,MMA845x::OS_NORMAL,MMA845x::HPF_OFF );

	/*
	 * Setup the Barometric sensor for post processed Ambient pressure, 4 samples per data acquisition.
	 * and a sample taken every second when in active mode
	 */
	evbBaro->setParameters(MPL3115A2::DATA_NORMAL, MPL3115A2::DM_BAROMETER, MPL3115A2::OR_16, MPL3115A2::AT_1);

	/*
	 * Setup the Ambient Light Sensor for continuous Ambient Light Sensing, 16 bit resolution,
	 * and 16000 lux range
	 */
	evbAmbLight->setMode(ISL29011::ALS_CONT);
	evbAmbLight->setResolution(ISL29011::ADC_16BIT);
	evbAmbLight->setRange(ISL29011::RNG_16000);

	/*
	 * Set the accelerometer for active mode
	 */
	evbAccel->activeMode();

	/*
	 * Clear the min-max registers in the Barometric Sensor
	 */
	evbBaro->clearMinMaxRegs();

	evbBackLight->setLEDCurrent(0);

	return Board_Ok;
}


EBoardStatus CBoardEVB::readSensors ( BoardSensorData &data )
{
    MMA845x_DATA accel_data;
    /*
     * Test Accelerometer XYZ data ready bit to see if acquisition complete
     */
    do {
        osDelay(100); // allows other threads to process
        result = evbAccel->getStatus();
    } while ((result & MMA845x::XYZDR) == 0 );

    /*
     * Retrieve and print out accelerometer data
     */
    accel_data = evbAccel->getXYZ();

    sprintf(txtstr,"Accelerometer");
    evbLCD->writeText(0,0,font_6x8,txtstr,strlen(txtstr));
    sprintf(txtstr, "x = %d", accel_data._x);
    evbLCD->writeText(20,1,font_6x8,txtstr,strlen(txtstr));
    sprintf(txtstr, "y = %d", accel_data._y);
    evbLCD->writeText(20,2,font_6x8,txtstr,strlen(txtstr));
    sprintf(txtstr, "z = %d", accel_data._z );
    evbLCD->writeText(20,3,font_6x8,txtstr,strlen(txtstr));

    sensorData.accel_x = accel_data._x;
    sensorData.accel_y = accel_data._y;
    sensorData.accel_z = accel_data._z;

    // Update accelerometer state
    evbLCD->startUpdate();
    evbLCD->clearBuffer();

    // convert to simple position value for use in send/recv
    if((accel_data._x > 500)&&(accel_data._z < 500))
    {
        if(position_value != 0x02)
            position_changed = true;
        position_value = 0x02;
    }
    else if((accel_data._x < 500)&&(accel_data._z > 500))
    {
        if(position_value != 0x01)
            position_changed = true;
        position_value = 0x01;
    }
    else
    {
        if(position_value != 0x00)
            position_changed = true;
        position_value= 0x00;
    }

    if(position_changed){
        evbBackLight->setLEDCurrent(0);
        // Turn LED off to indicate server not in agreement
        SYNC_LED=SYNC_LED_OOS;
        // Set reflected_value to an out of range value to stay
        // in fast transmit mode until server responds
        reflected_value = 0;
    }

    /*
     * Trigger a Pressure reading
     */
    evbBaro->setParameters(MPL3115A2::DATA_NORMAL, MPL3115A2::DM_BAROMETER, MPL3115A2::OR_16,
                           MPL3115A2::AT_1);
    evbBaro->triggerOneShot();

    /*
     * Test barometer device status to see if acquisition is complete
     */
    do {
        osDelay(100);			// allows other threads to process
        result = evbBaro->getStatus();
    } while ((result & MPL3115A2::PTDR) == 0 );

    /*
     * Retrieve and print out barometric pressure
     */
    pressure = evbBaro->getBaroData() >> 12; // convert 32 bit signed to 20 bit unsigned value
    num_whole = pressure >> 2;			// 18 bit integer significant
    num_frac = (pressure & 0x3) * 25;		// 2 bit fractional  0.25 per bit
    sensorData.pressure = pressure + (.25 * num_frac);

    sprintf(txtstr,"Press=%ld.%02d Pa", num_whole, num_frac);
    evbLCD->writeText(0,4,font_6x8,txtstr,strlen(txtstr));

    /*
     * Trigger a Altitude reading
     */
    evbBaro->setParameters(MPL3115A2::DATA_NORMAL, MPL3115A2::DM_ALTIMETER, MPL3115A2::OR_16,
                           MPL3115A2::AT_1);
    evbBaro->triggerOneShot();

    /*
     * Test barometer device status to see if acquisition is complete
     */
    do {
        osDelay(100);			// allows other threads to process
        result = evbBaro->getStatus();
    } while ((result & MPL3115A2::PTDR) == 0 );

    /*
     * Retrieve and print out altitude and temperature
     */
    baro_data = evbBaro->getAllData(false);
    baro_data._baro /= 4096;				// convert 32 bit signed to 20 bit signed value
    num_whole = baro_data._baro / 16;		//	18 bit signed significant integer
    num_frac = (baro_data._baro & 0xF) * 625 / 100;		// 4 bit fractional .0625 per bit
    sprintf(txtstr,"Alti=%ld.%03d m", num_whole, num_frac);
    evbLCD->writeText(0,5,font_6x8,txtstr,strlen(txtstr));
    num_whole = baro_data._temp / 16;		// 8 bit signed significant integer
    num_frac = (baro_data._temp & 0x0F) * 625 / 100;		// 4 bit fractional .0625 per bit
    sensorData.temperature = num_whole  + ((float)num_frac / 100);
    sprintf(txtstr,"Temp=%ld.%03d C", num_whole, num_frac);
    evbLCD->writeText(0,6,font_6x8,txtstr,strlen(txtstr));

    /*
     * retrieve and print out Ambient Light level
     */
    lux_data = evbAmbLight->getData();
    num_whole = lux_data * 24 / 100;		// 16000 lux full scale .24 lux per bit
    num_frac = lux_data * 24 % 100;
    sprintf(txtstr, "Light=%ld.%02d lux", num_whole, num_frac );
    evbLCD->writeText(0,7,font_6x8,txtstr,strlen(txtstr));

    evbLCD->endUpdate();

    return Board_Ok;
}


void BoardStop()
{
    evbBaro->triggerOneShot();
    do {
        osDelay(200);			// allows other threads to process
        result = evbBaro->getStatus();
    } while ((result & MPL3115A2::PTDR) == 0 );

    baro_data = evbBaro->getAllData(true);
    evbLCD->clearBuffer();
    sprintf(txtstr,"Exiting Program");
    evbLCD->writeText(0,4,font_6x8,txtstr,strlen(txtstr));
}



bool BoardCheckForExit( bool exit )
{
	// Check for PB1 press during network join attempt
	if (exit_program)
	{
		printf("Exiting program\n\r");
		evbLCD->clearBuffer();
		sprintf(txtstr,"Exiting Program");
		evbLCD->writeText(0,4,font_6x8,txtstr,strlen(txtstr));

		if(_exit)
			exit(1);
	}
	return false;
}

/*
 * Sets pb1_low flag. Slag is cleared in pb1_debounce thread
 */
void pb1ISR(void)
{
    if (!pb1_low)
        pb1_low = true;
}

/*
 * Debounces pb1. Also exits program if pushbutton 1 is pressed
 */
void pb1_debounce(void const *args)
{
    static uint8_t count = 0;

    while (true) {
        if (pb1_low && (mDot08 == 0))
            count++;
        else {
            count = 0;
            pb1_low = false;
        }

        if (count == 5)
            exit_program = true;

        Thread::wait(5);
    }
}

/*
 * Sets pb2_low flag. Flag is cleared in pb2_debounce thread
 */
void pb2ISR(void)
{
    if (!pb2_low)
        pb2_low = true;
}

/*
 * Debounces pb2. Also changes packet transmit time to every other,
 * every fifth, or every tenth sample when SW2 pushed
 * Also triggers a thread to transmit a configuration packet
 */
void pb2_debounce(void const *args)
{
    static uint8_t count = 0;

    while (true) {

        if (pb2_low && (mDot09 == 0))
            count++;
        else {
            count = 0;
            pb2_low = false;
        }

        if (count == 5){
            if (pckt_time >= 5)
                pckt_time /= 2;
            else
                pckt_time = 20;

            //thread_3->signal_set(0x10);		// signal config_pkt_xmit to send packet
            position_changed = true;
        }
        Thread::wait(5);
    }
 }

/*
 * Thread that is triggered by SW2 ISR. Sends a packet to the LoRa server with the new Packet Transmission time setting
 */
void config_pkt_xmit (void const *args)
{
    int32_t mdot_ret;

    std::vector<uint8_t> data;

    while (true) {
        Thread::signal_wait(0x10);		// wait for pb2ISR to signal send
        data.clear();
        data.push_back(0x0F);			// key for Configuration data (packet transmission timer)
        data.push_back(pckt_time);

        if ((mdot_ret = mdot_radio->send(data)) != mDot::MDOT_OK) {
            log_error(mdot_radio, "failed to send config data", mdot_ret);
        } else {
            printf("sent config data to gateway\r\n");
        }
    }
}

#endif
